import java.io.*;
import java.nio.channels.FileChannel;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class LabelEvents {

	/**
	 * @param args
	 * @throws FileNotFoundException 
	 */
	private class Record{
		int label;
		int hr;
		int min;
		int sec;
		public Record(int label, int hr, int min, int sec){
			this.label = label;
			this.hr = hr;
			this.min = min;
			this.sec = sec;
		}
	}
	private class Event{
		int label;
		File file;
		public Event(int label, File file){
			this.label = label;
			this.file = file;
		}
	}
	
	public void run() throws IOException{
		String[] users = new String[]{"angela", "catherine", "denise", "Ginette", "jacqueline", "jessica", "nahid" };
//		String[] users = new String[]{"angela"};
				
		for(String user: users){
			System.out.println("processing user: " + user);
			
			String data_folder = "/home/ml/hyuen/Documents/courses/machine learning/project/WSTdata/" + user + "/";
			String numerous_file_path = data_folder + "numeros.txt";			
			String output_file_folder = "/home/ml/hyuen/Documents/courses/machine learning/project/ws_code/experiment/identified_data/" + user + "/";
			String output_labels_file_path = output_file_folder + "labels.txt";
			
			new File(output_file_folder).mkdirs();
			
			InputStream f = new FileInputStream(numerous_file_path);
			Scanner sc = new Scanner(f);
					
			ArrayList<Record> records = new ArrayList<Record>();
			while(sc.hasNextLine()){
				String[] lines = sc.nextLine().split("\t");			
				int label = Integer.parseInt(lines[0]);
				int sublabel = Integer.parseInt(lines[1]);
				int h = Integer.parseInt(lines[2]);
				int min = Integer.parseInt(lines[3]);
				int sec = Integer.parseInt(lines[4]);			
				
				label = label * 10 + sublabel;
				
				if (label > 0) {				
					records.add( new Record(label, h, min, sec) );
				}
			}
			
			FileFilter filefilter = new FileFilter() {
		        public boolean accept(File file) {
		        	return (file.getName().endsWith(".dat"));		            
		        }
		    };
			
			String pattern = ".*_(\\d+)h(\\d+)m(\\d+)s\\.dat";
			File[] files= new File(data_folder).listFiles(filefilter);
			Arrays.sort( files );			
			int[] counter = new int[records.size()];
			ArrayList<Event> goodEvents = new ArrayList<Event>();
			for (File child : files ) {			
				Pattern r = Pattern.compile(pattern);
				Matcher m = r.matcher(child.getName());
				if (m.find( )) {
					int hr = Integer.parseInt(m.group(1));
					int min = Integer.parseInt(m.group(2));
					int s = Integer.parseInt(m.group(3));
					
					boolean matched = false;
					int label = -1;					
					for(int i = 0; i < records.size(); i++){
						Record record = records.get(i);					
						if( hr == record.hr && min == record.min && s == record.sec){
							if(matched){
//								System.out.println("ERROR double matching time: " + child);
								label = -1;
							}
							else{
								if( counter[i] > 0){
									System.out.println("ERROR Duplicated Matched Record.");
								}							
								counter[i]++;
								matched = true;
								label = record.label;
							}
						}
					}				
					if(label != -1){
						goodEvents.add( new Event(label, child));
					}
				} 			
			}		
			System.out.println("# of events for " + user + ": " + files.length + ", good ones: " + goodEvents.size());
			
			Collections.sort(goodEvents, new Comparator<Event>(){
				public int compare(Event o1, Event o2) {
					if(o1.label < o2.label) return -1;
					if(o1.label == o2.label && o1.file.getName().compareTo(o2.file.getName()) < 0 ) return -1;
					return 1;
				}				
			});
			
			PrintStream out = new PrintStream(new FileOutputStream(output_labels_file_path));
			for(int i = 0; i < goodEvents.size(); i++){
				Event event = goodEvents.get(i);

				FileChannel source = null;
				FileChannel destination = null;
				try {
					source = new FileInputStream( event.file.getPath() ).getChannel();
					destination = new FileOutputStream( output_file_folder + "event" + (i+1) + ".dat" ).getChannel();					
			        destination.transferFrom(source, 0, source.size());
			        source.close();
			        destination.close();
			    }
			    catch(Exception ex) {
			    	System.out.println("Transferring file failed: " + ex.toString());
			    	source.close();
			        destination.close();
			    }
				out.println(event.label);
			}			
			out.close();
			
			int cur = 0;
			while( cur < goodEvents.size()){
				int cur2 = cur;
				while( cur2 < goodEvents.size()-1 && goodEvents.get(cur2+1).label == goodEvents.get(cur).label )
					cur2++;
				
				String event_type_events_file_path = output_file_folder + "event_type" + goodEvents.get(cur).label + ".txt"; 
				PrintStream out2 = new PrintStream(new FileOutputStream(event_type_events_file_path));
				for(int i = cur; i <= cur2; i++)
					out2.println( i+1 );
				out2.close();				
				
				cur = cur2+1;
			}
			
			
			sc.close();
		}
	}
	
	public static void main(String[] args) throws IOException {
		// TODO Auto-generated method stub
		new LabelEvents().run();
	}

}

