%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;
constants;

%CLUSTERING_SIZE_MAX = CLUSTERING_SIZE_MIN;
%TOPIC_SIZE_MAX = TOPIC_SIZE_MIN;

cluster_range = (CLUSTERING_SIZE_MAX-CLUSTERING_SIZE_MIN)/CLUSTERING_SIZE_INT + 1;

build_all_folder;

purity = [];
randIndex = [];
recall = [];
precision = [];

training_sizes = sizes(is_testing_event == 0);
training_compressed_labels = compressed_labels(is_testing_event==0);

for c=CLUSTERING_SIZE_MIN:CLUSTERING_SIZE_INT:CLUSTERING_SIZE_MAX
	disp( strcat('cluster size', 32, int2str(c), ' ...') );

	purity_t = [];
	randIndex_t = [];
	precision_t = [];
	recall_t = [];
	for t=TOPIC_SIZE_MIN:TOPIC_SIZE_INT:TOPIC_SIZE_MAX
		training_data = load(strcat(all_lda_data_folder, 'c', int2str(c), 't', int2str(t), '_training.txt'));		

		for i=1:length(training_sizes)
			training_data(i,:) = training_data(i,:) * training_sizes(i);
		end

		%% compute Purity
		topic_labels_cnt =  zeros(t, length(unique_labels));
		for i=1:length(training_sizes)
			cet = training_compressed_labels(i);
			for j=1:t
				topic_labels_cnt(j, cet) = topic_labels_cnt(j, cet) + training_data(i,j);
			end
		end
		purity_t = [purity_t sum(max(topic_labels_cnt, [], 2)) / sum(topic_labels_cnt(:))];

		%% compute Rand Index		
		TP = 0;
		TN = 0;
		FN = 0;
		FP = 0;
		for i=1:length(unique_labels)
			for i2=1:length(unique_labels)
				for j=1:t
					for j2=1:t
						if i==i2 && j==j2
							TP = TP + (topic_labels_cnt(j,i) * (topic_labels_cnt(j,i)-1)) /2;
						end
						if i==i2 && j~=j2
							FN = FN + (topic_labels_cnt(j,i) * topic_labels_cnt(j2,i2)) /2;
						end
						if i~=i2 && j==j2
							FP = FP + (topic_labels_cnt(j,i) * topic_labels_cnt(j2,i2)) /2;
						end
						if i~=i2 && j~=j2
							TN = TN + (topic_labels_cnt(j,i) * topic_labels_cnt(j2,i2)) /2;
						end
					end
				end
			end
		end	

		randIndex_t = [randIndex_t (TP + TN) / (TP+TN+FP+FN)];
		precision_t = [precision_t TP / (TP + FP)];
		recall_t = [recall_t TP / (TP + FN)];		
	end
	purity = [purity; purity_t];
	randIndex = [randIndex; randIndex_t];
	precision = [precision; precision_t];
	recall = [recall; recall_t];			
end

save(strcat(all_lda_results_folder, 'purity.txt'), 'purity', '-ASCII', '-tabs');
save(strcat(all_lda_results_folder, 'randIndex.txt'), 'randIndex', '-ASCII', '-tabs');
save(strcat(all_lda_results_folder, 'recall.txt'), 'recall', '-ASCII', '-tabs');
save(strcat(all_lda_results_folder, 'precision.txt'), 'precision', '-ASCII', '-tabs');

